import React, { FC } from 'react';
import { useForm } from 'react-hook-form';
import { useTranslation } from 'react-i18next';
import {
  Box,
  Flex,
  FormControl,
  FormLabel,
  Input,
  VStack,
} from '@chakra-ui/react';

import { CarBodySelect } from './car-body';
import { CarColorInput } from './car-color';
import { CarNumberInput } from './car-number';
import { FormInputField, FormControllerField } from './field';
import { OrderFormValues } from './types';
import { PhoneInput } from './phone';
import { SubmitButton } from './submit';
import { defaultValues, onSubmit, useGetValidationRules } from './helper';
import { DateTimeInput } from './date-time';

export const OrderForm: FC = () => {
  const {
    handleSubmit,
    control,
    formState: { errors, isSubmitting },
    watch,
  } = useForm<OrderFormValues>({ defaultValues, shouldFocusError: true });

  const { t } = useTranslation('~', {
    keyPrefix: 'dry-wash.order-create.form',
  });

  const RULES = useGetValidationRules();

  const [availableDatetimeBegin, availableDatetimeEnd] = watch([
    'availableDatetimeBegin',
    'availableDatetimeEnd',
  ]);

  return (
    <Box p={4} marginInline='auto'>
      <VStack
        as='form'
        noValidate
        onSubmit={handleSubmit(onSubmit)}
        align='flex-start'
        gap={6}
      >
        <FormControllerField
          control={control}
          name='phone'
          label={t('phone-field.label')}
          isRequired
          rules={RULES.isValidPhoneNumber}
          errors={errors}
          Controller={PhoneInput}
        />
        <FormInputField
          control={control}
          name='carNumber'
          label={t('car-number-field.label')}
          isRequired
          rules={RULES.isValidCarNumber}
          errors={errors}
          Input={CarNumberInput}
        />
        <FormInputField
          control={control}
          name='carColor'
          label={t('car-color-field.label')}
          errors={errors}
          Input={CarColorInput}
        />
        <FormInputField
          control={control}
          name='carBody'
          label={t('car-body-field.label')}
          isRequired
          errors={errors}
          Input={CarBodySelect}
        />
        <FormInputField
          control={control}
          name='carLocation'
          label={t('washing-location-field.label')}
          isRequired
          errors={errors}
          Input={Input}
          help={t('washing-location-field.help')}
        />
        <FormControl isRequired>
          <FormLabel>{t('washing-datetime-field.label')}</FormLabel>
          <Flex flexWrap='wrap' gap={4}>
            <Box flex='1 0 100px'>
              <FormInputField
                control={control}
                name='availableDatetimeBegin'
                isRequired
                errors={errors}
                Input={DateTimeInput}
                inputProps={{
                  max: availableDatetimeEnd,
                }}
              />
            </Box>
            <Box flex='1 0 100px'>
              <FormInputField
                control={control}
                name='availableDatetimeEnd'
                isRequired
                errors={errors}
                Input={DateTimeInput}
                inputProps={{
                  min: availableDatetimeBegin,
                }}
              />
            </Box>
          </Flex>
        </FormControl>
        <SubmitButton isLoading={isSubmitting} mt={4} />
      </VStack>
    </Box>
  );
};

// todo: remove layout shift, when a validation message is displayed
// todo: select location using an interactive map
// todo: fix time range available values